@@ -95,11 +95,11 @@ class S3PutFileUploader(S3FileUploader):
             raise sdk_exceptions.S3UploadFailedError(response)
 
 
-class S3ProjectFileUploader(object):
-    def __init__(self, api_key, s3uploader=None, logger=None, ps_client_name=None):
+class ExperimentFileUploader(object):
+    def __init__(self, api_key, uploader=None, logger=None, ps_client_name=None):
         """
         :param str api_key:
-        :param S3FileUploader s3uploader:
+        :param S3FileUploader uploader:
         :param Logger logger:
         """
         self.logger = logger or MuteLogger()
@@ -109,7 +109,7 @@ class S3ProjectFileUploader(object):
             logger=self.logger,
             ps_client_name=ps_client_name,
         )
-        self.s3uploader = s3uploader or S3FileUploader(logger=self.logger, ps_client_name=ps_client_name)
+        self.uploader = uploader or S3FileUploader(logger=self.logger, ps_client_name=ps_client_name)
 
     def upload(self, file_path, project_id, cluster_id=None):
         """Upload file to S3 bucket for a project
@@ -121,9 +121,9 @@ class S3ProjectFileUploader(object):
         :rtype: str
         :return: S3 bucket's URL
         """
-        url, bucket_name, s3_fields = self._get_upload_data(file_path, project_id, cluster_id=cluster_id)
-        self.s3uploader.upload(file_path, url, s3_fields)
-        bucket_url = self.s3uploader.get_bucket_url(bucket_name, s3_fields)
+        url, bucket_name, fields = self._get_upload_data(file_path, project_id, cluster_id=cluster_id)
+        self.uploader.upload(file_path, url, fields)
+        bucket_url = self.uploader.get_bucket_url(bucket_name, fields)
         return bucket_url
 
     def _get_upload_data(self, file_path, project_handle, cluster_id=None):
@@ -158,11 +158,18 @@ class S3ProjectFileUploader(object):
 
             url = response_data["data"]["url"]
             bucket_name = response_data["data"]["bucket_name"]
-            s3_fields = response_data["data"]["fields"]
+            fields = response_data["data"]["fields"]
         except (KeyError, ValueError):
             raise sdk_exceptions.PresignedUrlMalformedResponseError("Response malformed")
 
-        return url, bucket_name, s3_fields
+        return url, bucket_name, fields
+
+
+class S3ProjectFileUploader(ExperimentFileUploader):
+    """
+    DEPRECATED: This class will be renamed to ExperimentFileUploader in release v0.8
+    """
+    pass
 
 
 class S3ModelFileUploader(object):
@@ -231,18 +238,40 @@ class S3ModelFileUploader(object):
         return client
 
 
-class S3WorkspaceDirectoryUploader(object):
+class S3ModelUploader(S3ModelFileUploader):
+    def upload(self, file_path, model_id):
+        if os.path.isdir(file_path):
+            file_path = self._zip_model_directory(file_path)
+
+        return super(S3ModelUploader, self).upload(file_path, model_id)
+
+    def _zip_model_directory(self, dir_path):
+        archiver = self._get_archiver()
+        archive_path = self._get_archive_path()
+        archiver.archive(dir_path, archive_path)
+        return archive_path
+
+    def _get_archiver(self):
+        return ZipArchiver()
+
+    def _get_archive_path(self):
+        archive_file_name = 'model.zip'
+        archive_file_path = os.path.join(tempfile.gettempdir(), archive_file_name)
+        return archive_file_path
+
+
+class ExperimentWorkspaceDirectoryUploader(object):
     def __init__(self, api_key, temp_dir=None, archiver=None, project_uploader=None, ps_client_name=None):
         """
         :param str api_key:
         :param str temp_dir:
         :param ZipArchiver archiver:
-        :param S3ProjectFileUploader project_uploader:
+        :param ExperimentFileUploader project_uploader:
         """
         self.temp_dir = temp_dir or tempfile.gettempdir()
         self.archiver = archiver or ZipArchiver()
         self.ps_client_name = ps_client_name
-        self.project_uploader = project_uploader or S3ProjectFileUploader(api_key, ps_client_name=ps_client_name)
+        self.project_uploader = project_uploader or ExperimentFileUploader(api_key, ps_client_name=ps_client_name)
 
     def upload(self, workspace_dir_path, project_id, exclude=None, temp_file_name="temp.zip"):
         """Archive and upload a workspace directory
@@ -263,3 +292,74 @@ class S3WorkspaceDirectoryUploader(object):
     def get_archive_path(self, temp_file_name):
         archive_path = os.path.join(self.temp_dir, temp_file_name)
         return archive_path
+
+
+class S3WorkspaceDirectoryUploader(ExperimentWorkspaceDirectoryUploader):
+    """
+    DEPRECATED: This class will be renamed to ExperimentWorkspaceDirectoryUploader in release v0.8
+    """
+    pass
+
+
+class DeploymentWorkspaceDirectoryUploader(object):
+    def __init__(self, api_key, uploader=None, logger=None, ps_client_name=None):
+        """
+        :param str api_key:
+        :param S3PutFileUploader uploader:
+        :param Logger logger:
+        """
+        self.logger = logger or MuteLogger()
+        self.ps_api_client = http_client.API(
+            config.CONFIG_HOST,
+            api_key=api_key,
+            logger=self.logger,
+            ps_client_name=ps_client_name,
+        )
+        self.uploader = S3PutFileUploader(logger=self.logger, ps_client_name=ps_client_name)
+
+    def _get_upload_data(self, file_path, project_id, cluster_id=None):
+        """Ask API for data required to upload deployment workspace a file to S3
+
+        :param str file_path:
+        :param str project_id:
+        :param str cluster_id:
+
+        :rtype: str
+        :return: URL to which send the file, name of the bucket and a dictionary required by S3 service
+        """
+        file_name = os.path.basename(file_path)
+        params = {
+            "fileName": file_name,
+            "contentType": mimetypes.guess_type(file_path)[0] or "",
+        }
+        if project_id:
+            params['projectId'] = project_id
+        if cluster_id:
+            params['clusterHandle'] = cluster_id
+        response = self.ps_api_client.get("/deployments/getPresignedDeploymentUrl", params=params)
+        if not response.ok:
+            raise sdk_exceptions.PresignedUrlConnectionError(response.reason)
+
+        try:
+            response_data = response.json()
+            presigned_url = response_data['presignedUrl']
+            bucket_name = response_data['bucketName']
+            workspace_url = response_data['workspaceUrl']
+        except (KeyError, ValueError):
+            raise sdk_exceptions.PresignedUrlMalformedResponseError("Response malformed")
+
+        return presigned_url, bucket_name, workspace_url
+
+    def upload(self, file_path, project_id=None, cluster_id=None, **kwargs):
+        """Upload file to S3 bucket for a project
+
+        :param str file_path:
+        :param str project_id:
+        :param str cluster_id:
+
+        :rtype: str
+        :return: S3 bucket's URL
+        """
+        url, bucket_name, workspace_url = self._get_upload_data(file_path, project_id, cluster_id)
+        self.uploader.upload(file_path, url)
+        return workspace_url
